#include "Animation.h"

Animation::Animation(int x, int y, const Surface & surface, 
	int frameWidth, int frameHeight, 
	int totalFrame, int stopFrame, int playFrame,
	float interval, Color chromaKey)
	:
	surface(surface),
	stopFrame(stopFrame),
	playFrame(playFrame),
	interval(interval),
	chromaKey(chromaKey)
{
	for (int i = 0; i < totalFrame; i++)
	{
		frames.emplace_back(
			x + i * frameWidth,
			x + (i + 1) * frameWidth,
			y * frameHeight,
			y * frameHeight + frameHeight
		);
	}
}

Animation::Animation(const Vei2 & position, const Surface & surface,
	int frameWidth, int frameHeight, 
	int totalFrame, int stopFrame, int playframe,
	float interval, Color chromaKey)
	: Animation(position.x, position.y, surface,
		frameWidth, frameHeight, 
		totalFrame, stopFrame, playframe,
		interval, chromaKey)
{
	
}

void Animation::Draw(const Vei2& position, Graphics & gfx) const
{
	gfx.DrawSprite(position.x, position.y, frames[currentFrame], 
		surface, SpriteEffect::Ghost(chromaKey, 0.5f));
}

void Animation::Draw(const Vei2& position, Graphics & gfx, const RectI & clip) const
{
	gfx.DrawSprite(position.x, position.y, frames[currentFrame], 
		clip, surface, SpriteEffect::Copy());
}

void Animation::Draw(const Vei2 & position, Graphics & gfx, 
	Color substituteColor) const
{
	gfx.DrawSprite(position.x, position.y, frames[currentFrame],
		surface, SpriteEffect::Substitution(chromaKey, substituteColor));
}

void Animation::Update(float dt)
{
	totalTime += dt;
	while (totalTime >= interval)
	{
		Next();
		totalTime -= interval;
	}
}

bool Animation::isStopped()
{
	return currentFrame == stopFrame;
}

void Animation::Stop()
{
	currentFrame = stopFrame;
}

void Animation::Play()
{
	currentFrame = playFrame;
}

void Animation::Next()
{
	currentFrame += 1;
	if (currentFrame >= frames.size())
	{
		currentFrame = playFrame;
	}
}
