#include "Link.h"

Link::Link(const Vec2 & in_position)
	:position(in_position)
{
	for (int i = 0; i < (int)Sequences::Count; i++)
	{
		animations.emplace_back(0, i, surface, 
			frameWidth, frameHeight, 
			totalFrame, stopFrame, playFrame,
			interval, chromaKey);
	}
}

void Link::SetDirection(const Vec2 & in_velocity)
{
	velocity = in_velocity;
	if (velocity.x > 0)
	{
		currentDirection = Sequences::Right;
	} 
	else if (velocity.x < 0)
	{
		currentDirection = Sequences::Left;
	}
	else if (velocity.y > 0)
	{
		currentDirection = Sequences::Down;
	}
	else if (velocity.y < 0)
	{
		currentDirection = Sequences::Up;
	}
	if (animations[(int)currentDirection].isStopped())
	{
		animations[(int)currentDirection].Play();
	}
}

void Link::Update(float dt)
{
	damage.Update(dt);
	if (IsStop())
	{
		animations[(int)currentDirection].Stop();
	}
	else
	{
		position += velocity * dt * speed;
		animations[(int)currentDirection].Update(dt);
	}
}

Effect & Link::GetEffect()
{
	return damage;
}



void Link::Draw(Graphics & gfx) const
{
	animations[(int)currentDirection].Draw((Vei2)position, gfx);
}

void Link::Draw(Graphics & gfx, Color substituteColor) const
{
	animations[(int)currentDirection].Draw((Vei2)position, gfx, substituteColor);
}

bool Link::IsStop()
{
	return velocity.x == 0.0f && velocity.y == 0.0f;
}
