#include "Font.h"
#include <assert.h>

Font::Font(const std::string & fileName, Color chromaKey)
	:surface(fileName),
	chromaKey(chromaKey)
{
	glyphWidth = surface.GetWidth() / numOfColumns;
	glyphHeight = surface.GetHeight() / numOfRows;

	assert(glyphWidth * numOfColumns == surface.GetWidth());
	assert(glyphHeight * numOfRows == surface.GetHeight());
}

void Font::DrawText(const std::string & text, Color substituteColor,
	const Vei2 position, Graphics & gfx) const
{
	//create effect funtor
	SpriteEffect::Substitution effect(chromaKey, substituteColor);

	Vei2 currentPosition = position;
	for (const char& c : text)
	{
		if (c == '\n')
		{
			currentPosition.x = position.x;
			//new line
			currentPosition.y += glyphHeight;
			continue;
		}
		RectI glyphRect = GetGlyph(c);
		//skip space, space no need to draw
		if (c >= firstChar + 1 && c <= lastChar)
		{
			gfx.DrawSprite(currentPosition.x, currentPosition.y,
				glyphRect, surface, effect);
		}
		currentPosition.x += glyphWidth;
	}
}

RectI Font::GetGlyph(char c) const
{
	assert(c >= firstChar && c <= lastChar);
	int index = c - ' ';

	int x = index % numOfColumns;
	int y = index / numOfColumns;

	int left = x * glyphWidth;
	int right = x * glyphWidth + glyphWidth;
	int top = y * glyphHeight;
	int bottom = y * glyphHeight + glyphHeight;

	return RectI(left, right, top, bottom);
}