#pragma once
#include "Graphics.h"
#include "Colors.h"
#include <assert.h>

namespace SpriteEffect
{
	class Substitution
	{
	public:
		Substitution(Color chroma, Color substituteColor)
			: chroma(chroma), substituteColor(substituteColor)
		{}

		void operator()(Color sourceColor, int x, int y, Graphics& gfx)
		{
			if (sourceColor != chroma)
			{
				gfx.PutPixel(x, y, substituteColor);
			}
		}
	private:
		Color chroma = Colors::Magenta;
		Color substituteColor;

	};

	class Chroma
	{
	public:
		Chroma(Color chroma)
			: chroma(chroma)
		{}

		void operator()(Color sourceColor, int x, int y, Graphics& gfx)
		{
			if (sourceColor != chroma)
			{
				gfx.PutPixel(x, y, sourceColor);
			}
		}
	private:
		Color chroma = Colors::Magenta;
	};

	class Ghost
	{
	public:
		Ghost(Color chroma, float colorValue)
			: chroma(chroma), colorValue(colorValue)
		{
			assert(colorValue >= 0.0f && colorValue <= 1.0f);
		}

		void operator()(Color sourceColor, int x, int y, Graphics& gfx)
		{
			if (sourceColor != chroma)
			{
				Color blend =
				{
					unsigned char(sourceColor.GetR() * colorValue),
					unsigned char(sourceColor.GetG() * colorValue),
					unsigned char(sourceColor.GetB() * colorValue)
				};
				gfx.PutPixel(x, y, blend);
			}
		}
	private:
		Color chroma = Colors::Magenta;
		float colorValue = 1.0f;
	};

	class Copy
	{
	public:
		void operator()(Color sourceColor, int x, int y, Graphics& gfx)
		{
			gfx.PutPixel(x, y, sourceColor);
		}
	private:
		Color chroma = Colors::Magenta;
	};
}
