#include "Surface.h"
#include <cassert>
#include "ChiliWin.h"
#include <fstream>

Surface::Surface(const std::string & filename)
{
	std::ifstream file(filename, std::ios_base::binary);
	assert(file);

	BITMAPFILEHEADER bmpFileHeader;
	file.read(reinterpret_cast<char*>(&bmpFileHeader), sizeof(bmpFileHeader));

	BITMAPINFOHEADER bmpInfoHeader;
	file.read(reinterpret_cast<char*>(&bmpInfoHeader), sizeof(bmpInfoHeader));

	const bool is32bit = bmpInfoHeader.biBitCount == 32;

	//Bits of the image. EX: 24 bits Image, 8 bits Image etc.
	assert(bmpInfoHeader.biBitCount == 24 || is32bit);
	//biCompression == BI_RGB (An uncompressed format)
	assert(bmpInfoHeader.biCompression == BI_RGB);

	//Load width and height of image 
	width = bmpInfoHeader.biWidth;
	height = abs(bmpInfoHeader.biHeight);

	pixels.resize(width * height);

	file.seekg(bmpFileHeader.bfOffBits);

	//padding: NƵc 4 bytes , 24 bits only
	const int padding = (4 - (width % 4)) % 4;

	int yStart;
	int yEnd;
	int increment;

	if (bmpInfoHeader.biHeight < 0)
	{
		//top-down
		yStart = 0;
		yEnd = height;
		increment = 1;
	}
	else
	{
		//bottom-up
		yStart = height - 1;
		yEnd = -1;
		increment = -1;
	}

	for (int y = yStart; y != yEnd; y+= increment)
	{
		for (int x = 0; x < width; x++)
		{
			//Color Order: BGRA
			PutPixel(x, y, Color(file.get(), file.get(), file.get()));
			if (is32bit)
			{
				//skip alpha
				file.seekg(1, std::ios::cur);
			}
		}
		if (!is32bit)
		{
			file.seekg(padding, std::ios::cur);
		}
		
	}
}

Surface::Surface(int width, int height)
	:
	width(width),
	height(height),
	pixels(width * height)
{

}

Surface::Surface(Surface&& donor)
{
	*this = std::move(donor);
}

Surface& Surface::operator=(Surface&& rhs)
{
	width = rhs.width;
	height = rhs.height;
	pixels = std::move(rhs.pixels);
	rhs.width = 0;
	rhs.height = 0;
	return *this;
}

int Surface::GetWidth() const
{
	return width;
}

int Surface::GetHeight() const
{
	return height;
}

RectI Surface::GetRect() const
{
	return { 0, width, 0, height };
}

const Color* Surface::Data() const
{
	return pixels.data();
}